@@ -2,9 +2,9 @@
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 from trytond.model import Workflow, ModelView, ModelSQL, fields
-from trytond.pyson import Eval, If
+from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.pool import Pool
 
 __all__ = ['Statement', 'Line']
@@ -19,13 +19,13 @@ class Statement(Workflow, ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
         select=True, states=_STATES, domain=[
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', 0)),
+                Eval('context', {}).get('company', -1)),
             ],
         depends=_DEPENDS)
     journal = fields.Many2One('account.statement.journal', 'Journal',
         required=True,
         domain=[
-            ('company', '=', Eval('context', {}).get('company', 0)),
+            ('company', '=', Eval('context', {}).get('company', -1)),
             ],
         states={
             'readonly': (Eval('state') != 'draft') | Eval('lines'),
@@ -59,8 +59,6 @@ class Statement(Workflow, ModelSQL, ModelView):
         ('cancel', 'Canceled'),
         ('posted', 'Posted'),
         ], 'State', readonly=True, select=True)
-    move_lines = fields.Function(fields.One2Many('account.move.line',
-        None, 'Move Lines'), 'get_move_lines')
 
     @classmethod
     def __setup__(cls):
@@ -73,6 +71,7 @@ class Statement(Workflow, ModelSQL, ModelView):
                 })
         cls._transitions |= set((
                 ('draft', 'validated'),
+                ('draft', 'cancel'),
                 ('validated', 'posted'),
                 ('validated', 'cancel'),
                 ('cancel', 'draft'),
@@ -88,12 +87,13 @@ class Statement(Workflow, ModelSQL, ModelView):
                     'invisible': Eval('state') != 'validated',
                     },
                 'cancel': {
-                    'invisible': Eval('state') != 'validated',
+                    'invisible': ~Eval('state').in_(['draft', 'validated']),
                     },
                 })
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
 
         # Migration from 1.8: new field company
@@ -180,26 +180,8 @@ class Statement(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        statements = cls.search(['OR',
-                ('start_balance',) + tuple(clause[1:]),
-                ('end_balance',) + tuple(clause[1:]),
-                ])
-        if statements:
-            return [('id', 'in', [s.id for s in statements])]
         return [('journal',) + tuple(clause[1:])]
 
-    def get_move_lines(self, name):
-        '''
-        Return the move lines that have been generated by the statement.
-        '''
-        move_lines = []
-        for line in self.lines:
-            if not line.move:
-                continue
-            for move_line in line.move.lines:
-                move_lines.append(move_line.id)
-        return move_lines
-
     def get_end_balance(self, name):
         end_balance = self.start_balance
         for line in self.lines:
@@ -225,27 +207,27 @@ class Statement(Workflow, ModelSQL, ModelView):
             invoice_id2amount_to_pay = {}
             for invoice in invoices:
                 with Transaction().set_context(date=invoice.currency_date):
-                    invoice_id2amount_to_pay[invoice.id] = (
+                    if invoice.type in ('out_invoice', 'in_credit_note'):
+                        sign = -1
+                    else:
+                        sign = 1
+                    invoice_id2amount_to_pay[invoice.id] = sign * (
                         Currency.compute(invoice.currency,
                             invoice.amount_to_pay, self.journal.currency))
 
             for line in self.lines or []:
                 if line.invoice and line.id:
                     amount_to_pay = invoice_id2amount_to_pay[line.invoice.id]
-                    if abs(line.amount) > amount_to_pay:
-                        res['lines'].setdefault('update', [])
-                        if self.journal.currency.is_zero(amount_to_pay):
-                            res['lines']['update'].append({
+                    res['lines'].setdefault('update', [])
+                    if (not self.journal.currency.is_zero(amount_to_pay)
+                            and (line.amount >= 0) == (amount_to_pay <= 0)):
+                        res['lines']['update'].append({
                                 'id': line.id,
-                                'invoice': None,
-                                })
-                        else:
-                            res['lines']['update'].append({
-                                'id': line.id,
-                                'amount': (amount_to_pay
-                                        if line.amount >= 0
-                                        else -amount_to_pay),
+                                'amount': (amount_to_pay.copy_sign(line.amount)
+                                    if abs(line.amount) > abs(amount_to_pay)
+                                    else line.amount)
                                 })
+                        if abs(line.amount) > abs(amount_to_pay):
                             res['lines'].setdefault('add', [])
                             vals = {}
                             for field_name, field in Line._fields.iteritems():
@@ -261,15 +243,19 @@ class Statement(Workflow, ModelSQL, ModelView):
                                 else:
                                     vals[field_name] = value
                             del vals['id']
-                            vals['amount'] = (abs(line.amount)
-                                - amount_to_pay)
-                            if line.amount < 0:
-                                vals['amount'] = - vals['amount']
+                            vals['amount'] = line.amount + amount_to_pay
                             vals['invoice'] = None
                             del vals['invoice.rec_name']
                             res['lines']['add'].append(vals)
-                    invoice_id2amount_to_pay[line.invoice.id] = \
-                        amount_to_pay - abs(line.amount)
+                            invoice_id2amount_to_pay[line.invoice.id] = 0
+                        else:
+                            invoice_id2amount_to_pay[line.invoice.id] = (
+                                line.amount + amount_to_pay)
+                    else:
+                        res['lines']['update'].append({
+                                'id': line.id,
+                                'invoice': None,
+                                })
         return res
 
     @classmethod
@@ -349,17 +335,15 @@ class Line(ModelSQL, ModelView):
     description = fields.Char('Description')
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
     invoice = fields.Many2One('account.invoice', 'Invoice',
+        on_change=['party', 'account', 'invoice'],
         domain=[
-            ('party', '=', Eval('party')),
-            ('account', '=', Eval('account')),
+            If(Bool(Eval('party')), [('party', '=', Eval('party'))], []),
+            If(Bool(Eval('party')), [('account', '=', Eval('account'))], []),
             If(Eval('_parent_statement', {}).get('state') == 'draft',
                 ('state', '=', 'posted'),
                 ('state', '!=', '')),
             ],
-        states={
-            'readonly': (~Eval('amount') | ~Eval('party') | ~Eval('account')),
-            },
-        depends=['party', 'account', 'amount'])
+        depends=['party', 'account'])
 
     @classmethod
     def __setup__(cls):
@@ -442,9 +426,24 @@ class Line(ModelSQL, ModelView):
                 res['invoice'] = None
         return res
 
+    def on_change_invoice(self):
+        changes = {}
+        if self.invoice:
+            if not self.party:
+                changes['party'] = self.invoice.party.id
+                changes['party.rec_name'] = self.invoice.party.rec_name
+            if not self.account:
+                changes['account'] = self.invoice.account.id
+                changes['account.rec_name'] = self.invoice.account.rec_name
+        return changes
+
     def get_rec_name(self, name):
         return self.statement.rec_name
 
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        return [('statement.rec_name',) + tuple(clause[1:])]
+
     @classmethod
     def copy(cls, lines, default=None):
         if default is None:
@@ -490,7 +489,7 @@ class Line(ModelSQL, ModelView):
                 amount_to_pay = Currency.compute(self.invoice.currency,
                     self.invoice.amount_to_pay,
                     self.statement.journal.currency)
-            if amount_to_pay < abs(self.amount):
+            if abs(amount_to_pay) < abs(self.amount):
                 lang, = Lang.search([
                         ('code', '=', Transaction().language),
                         ])
@@ -506,7 +505,7 @@ class Line(ModelSQL, ModelView):
                     self.amount, self.statement.company.currency)
 
             reconcile_lines = self.invoice.get_reconcile_lines_for_amount(
-                abs(amount))
+                amount)
 
             for move_line in move.lines:
                 if move_line.account == self.invoice.account:
